`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2022/05/17 23:16:57
// Design Name: 
// Module Name: CounterTest
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module CPLD_Transmitter(
    Clk, BckIn, LrckIn, DataIn, 
	 BckOut, LrckOut, DataOut, StartFlag
    );
    input Clk, BckIn, LrckIn, DataIn;
	 input StartFlag;
	 output BckOut, LrckOut, DataOut;
	 
	 ///// In Part
	 reg RegLrckIn = 0;
	 reg RegDataIn = 0;
	 reg DinWen = 0;
	 reg DinWenDelay = 0;
	 reg [19:0] DinStateCounter = 0;
	 reg [1:0] DinState = 0;	 
	 reg [5:0] DinInnerCounter = 0;
	 
	 ///// Out Part
	 reg [1:0] BckOutCounter = 0;
	 reg [5:0] LrckOutCounter = 0;
	 reg FifoRen = 0;
	 reg FifoRenPos = 0;	
	 reg DataOut = 0;
	 reg [6:0] InnerInputCounter=0;
	 reg [4:0] DoutReadCounter = 0;
	 reg [1:0] ReadState = 0;
	 reg [1:0] ReadStatePos = 0;
	 reg [31:0] FifoShifter = 0;
	 reg [19:0] FifiReadOutterCounter = 0;
	 reg CaptureTrigState = 0;
	 reg CaptureTrigStateOut = 0;
//	 reg ReadStart = 0;
	 wire [5:0] FifoWord;
	 wire FifoEmp;
	 wire FifoReadOut;
	 wire [22:0] WireForTrig;
    
	 assign BckOut = BckOutCounter[1];
	 assign LrckOut = LrckOutCounter[5];
	 assign WireForTrig = 23'b 10011011001000100100001 ;
	 
	 //系统常量
	 parameter LrckCounter99 = (1006000 - 1);
	 parameter LrckCounter98 = (1006000 - 2);
	 parameter LrckCounter97 = (1006000 - 3);
	 
	 ///// In Part Logic
	 always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					RegLrckIn <= 1'b0;
					RegDataIn <= 1'b0;
				end
			else
				begin
					RegLrckIn <= LrckIn;
					RegDataIn <= DataIn;
				end
		end
		 
   always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					InnerInputCounter <= 7'd62;
				end
			else
				begin
					if((DinState==2'd2)||(DinState==2'd0))
						begin
							if(InnerInputCounter < 7'd62)
								begin
									InnerInputCounter <= InnerInputCounter + 7'd1;
								end
							else
								begin
									if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
										InnerInputCounter <= 7'd0;
								end
						end
					else
						InnerInputCounter <= 7'd62;
				end
		end	
	
	always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					DinInnerCounter <= 6'd0;
				end
			else
				begin
					if(DinState==2'd2)
						begin							
							if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
								DinInnerCounter <= 6'd0;
							else	
								DinInnerCounter <= DinInnerCounter + 5'd1;
						end
					else
						DinInnerCounter <= 6'd0;
				end
		end
		
	always@(negedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					DinWenDelay <= 5'd0;
				end
			else
				begin
					DinWenDelay <= DinWen;
				end
		end	
		
    always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					DinState <= 2'd0;
				end
			else
				begin
					case(DinState)
						2'd0:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
									begin
										DinState <= 2'd2;
									end
							end
						2'd1:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
									begin
										DinState <= 2'd2;
									end
							end
						2'd2:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) && ( DinStateCounter >= LrckCounter98 ) )
									begin
										DinState <= 2'd3;
									end
							end			
						2'd3:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) && ( DinStateCounter >= LrckCounter99 ) )
									begin
										DinState <= 2'd1;
									end
							end
									
					endcase				
				end
		end		
	
	always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					DinStateCounter <= 20'd0;
				end
			else
				begin
					if(DinState == 2'd2 || DinState == 2'd3)
						begin
							if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) && ( DinStateCounter >= LrckCounter99 ) )	
								DinStateCounter <= 20'd0;
							else if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) && (InnerInputCounter == 7'd62))
								begin
									DinStateCounter <= DinStateCounter + 20'd1;
								end
							else
								DinStateCounter <= DinStateCounter;
						end
					else
						DinStateCounter <= 20'd0;
				end
		end
		
	always@(posedge BckIn)
		begin
			if(StartFlag == 1'b0)
				begin
					DinWen <= 1'd0;
				end
			else
				begin
					case(DinState)
						2'd0:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
									begin
										DinWen <= 1'd1;
									end
								else if( (DinStateCounter >= LrckCounter98) )
									begin
										DinWen <= 1'd0;
									end
							end
						2'd1:
							begin
								if( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) )
									begin
										DinWen <= 1'd1;
									end
								else if( (DinStateCounter >= LrckCounter98) )
									begin
										DinWen <= 1'd0;
									end	
							end
						2'd2:
							begin
								if( (DinStateCounter >= LrckCounter98)  || (DinInnerCounter == 5'd24))
									begin
										DinWen <= 1'd0;
									end
								else if((DinStateCounter <= LrckCounter98) && ( (RegLrckIn == 1'b1) && (LrckIn == 1'b0) ) )
										DinWen <= 1'd1;
							end
						2'd3:
							begin
								if( (DinStateCounter >= LrckCounter98) )
									begin
										DinWen <= 1'd0;
									end	
							end
					endcase	
				end
		end
		

	IP_FIFO IP_FIFO(
	.aclr(!StartFlag),
	.data(RegDataIn),
	.rdclk(BckOut),
	.rdreq(FifoRen),
	.wrclk(BckIn),
	.wrreq(DinWenDelay),
	.q(FifoReadOut),
//	.rdempty(FifoEmp),
	.rdusedw(FifoWord)
	);		
	
	///// Out Part Logic	
	
	always@(posedge Clk)
		begin
			BckOutCounter <= BckOutCounter + 2'd1;
		end	

	always@(negedge BckOut)
		begin
			LrckOutCounter <= LrckOutCounter + 6'd1;
		end	
		
	always@(negedge BckOut)
		begin
			if((LrckOutCounter==6'd63) || (LrckOutCounter==6'd31))
				DoutReadCounter <= 5'd0;
			else
				DoutReadCounter <= DoutReadCounter + 5'd1;
		end
		
	always@(posedge BckOut)
		begin
			FifoRenPos <= FifoRen;
		end			
		
	always@(negedge BckOut)
		begin
			if( (LrckOutCounter == 6'd62) && (FifoWord >= 6'd6) && (ReadStatePos == 2'd0) )
				FifoRen <= 1'd1;
			else if( (LrckOutCounter == 6'd62) && (ReadStatePos == 2'd1) )	
				FifoRen <= 1'd1;
			else if( (LrckOutCounter == 6'd23) && (FifoRenPos == 1'd1)  )	// && (ReadStatePos == 2'd1)
				FifoRen <= 1'd0;		
		end
		
	always@(posedge BckOut)
		begin
			if(StartFlag == 1'b0)
				begin
					ReadStatePos <= 2'd0;
				end
			else
				begin
					ReadStatePos <= ReadState;
				end
		end
	
	always@(posedge BckOut)
		begin
			if(StartFlag == 1'b0)
				begin
					FifiReadOutterCounter <= 20'd0;
				end
			else
				begin
					if(ReadStatePos == 1'b0)
						FifiReadOutterCounter <= 20'd0;
					else
						begin
							if( (LrckOutCounter == 6'd62) && (FifiReadOutterCounter < LrckCounter99))
								FifiReadOutterCounter <= FifiReadOutterCounter + 20'd1;
							else
								FifiReadOutterCounter <= FifiReadOutterCounter;
						end
				end
		end	
		
		
	always@(negedge BckOut)
		begin
			if(StartFlag == 1'b0)
				begin
					ReadState <= 2'd0;
				end
			begin	
				if( (LrckOutCounter == 6'd62) && (FifoWord >= 6'd6) )
					ReadState <= 2'd1;
				else if( (FifoWord <= 6'd1) && (ReadState == 2'd1) && (FifiReadOutterCounter >= LrckCounter99 )	)
					ReadState <= 2'd0;
			end
		end
		
	always@(negedge BckOut)
		begin
			FifoShifter <= {FifoShifter[29:0], DataOut};
		end
	
	always@(negedge BckOut)
		begin
			if(ReadStatePos == 1'b0)
				DataOut <= 1'b0;
			else
				begin
					if(FifoRenPos == 1'b1)
						DataOut <= FifoReadOut;
					else
						DataOut <= FifoShifter[30];
				end
		end	
		
	always@(posedge BckOut)
		begin
			if(StartFlag == 1'b0)
				begin
					CaptureTrigState <= 1'd0;
				end
			begin	
				if(FifoRen == 1'b1)
					begin
						case(LrckOutCounter)
							6'd0:
								begin
									CaptureTrigState <= 1'd0;
								end
							6'd1:	
								begin
									if(DataOut!=WireForTrig[22])
										CaptureTrigState <= 1'd1;
								end
							6'd2:	
								begin
									if(DataOut!=WireForTrig[21])
										CaptureTrigState <= 1'd1;
								end	
							6'd3:	
								begin
									if(DataOut!=WireForTrig[20])
										CaptureTrigState <= 1'd1;
								end	
							6'd4:	
								begin
									if(DataOut!=WireForTrig[19])
										CaptureTrigState <= 1'd1;
								end
							6'd5:	
								begin
									if(DataOut!=WireForTrig[18])
										CaptureTrigState <= 1'd1;
								end
							6'd6:	
								begin
									if(DataOut!=WireForTrig[17])
										CaptureTrigState <= 1'd1;
								end
							6'd7:	
								begin
									if(DataOut!=WireForTrig[16])
										CaptureTrigState <= 1'd1;
								end
							6'd8:	
								begin
									if(DataOut!=WireForTrig[15])
										CaptureTrigState <= 1'd1;
								end
							6'd9:	
								begin
									if(DataOut!=WireForTrig[14])
										CaptureTrigState <= 1'd1;
								end
							6'd10:	
								begin
									if(DataOut!=WireForTrig[13])
										CaptureTrigState <= 1'd1;
								end
							6'd11:	
								begin
									if(DataOut!=WireForTrig[12])
										CaptureTrigState <= 1'd1;
								end
							6'd12:	
								begin
									if(DataOut!=WireForTrig[11])
										CaptureTrigState <= 1'd1;
								end
							6'd13:	
								begin
									if(DataOut!=WireForTrig[10])
										CaptureTrigState <= 1'd1;
								end
							6'd14:	
								begin
									if(DataOut!=WireForTrig[9])
										CaptureTrigState <= 1'd1;
								end
							6'd15:	
								begin
									if(DataOut!=WireForTrig[8])
										CaptureTrigState <= 1'd1;
								end
							6'd16:	
								begin
									if(DataOut!=WireForTrig[7])
										CaptureTrigState <= 1'd1;
								end
							6'd17:	
								begin
									if(DataOut!=WireForTrig[6])
										CaptureTrigState <= 1'd1;
								end
							6'd18:	
								begin
									if(DataOut!=WireForTrig[5])
										CaptureTrigState <= 1'd1;
								end
							6'd19:	
								begin
									if(DataOut!=WireForTrig[4])
										CaptureTrigState <= 1'd1;
								end
							6'd20:	
								begin
									if(DataOut!=WireForTrig[3])
										CaptureTrigState <= 1'd1;
								end
							6'd21:	
								begin
									if(DataOut!=WireForTrig[2])
										CaptureTrigState <= 1'd1;
								end
							6'd22:	
								begin
									if(DataOut!=WireForTrig[1])
										CaptureTrigState <= 1'd1;
								end							
						endcase
					end
				else
					CaptureTrigState <= 1'd0;
			end
		end

	always@(posedge BckOut)
		begin
			if(StartFlag == 1'b0)
				begin
					CaptureTrigStateOut <= 1'd0;
				end
			else
				begin
					if( (FifoRen == 1'b1) && (LrckOutCounter == 6'd23) && (CaptureTrigState == 1'b1) && (FifiReadOutterCounter<=LrckCounter97) && (FifiReadOutterCounter > 20'd0)) 
						CaptureTrigStateOut <= 1'd1;
					else
						CaptureTrigStateOut <= 1'd0;
				end
		end				
	
endmodule
